@@ -65,7 +65,6 @@
 #include <meta/window.h>
 #include "compositor-private.h"
 #include <meta/compositor-mutter.h>
-#include "xprops.h"
 #include <meta/prefs.h>
 #include <meta/main.h>
 #include <meta/meta-background-actor.h>
@@ -77,14 +76,16 @@
 #include "display-private.h" /* for meta_display_lookup_x_window() */
 #include "util-private.h"
 #include "frame.h"
-#include "meta-wayland-private.h"
-#include "meta-wayland-pointer.h"
-#include "meta-wayland-keyboard.h"
 #include <X11/extensions/shape.h>
 #include <X11/extensions/Xcomposite.h>
 
-/* #define DEBUG_TRACE g_print */
-#define DEBUG_TRACE(X)
+#include "wayland/meta-wayland-private.h"
+
+static gboolean
+is_modal (MetaDisplay *display)
+{
+  return display->grab_op == META_GRAB_OP_COMPOSITOR;
+}
 
 static inline gboolean
 composite_at_least_version (MetaDisplay *display, int maj, int min)
@@ -109,13 +110,8 @@ meta_finish_workspace_switch (MetaCompScreen *info)
   for (l = info->windows; l; l = l->next)
     meta_window_actor_sync_visibility (l->data);
 
-  /*
-   * Fix up stacking order in case the plugin messed it up.
-   */
+  /* Fix up stacking order. */
   sync_actor_stacking (info);
-
-/*   printf ("... FINISHED DESKTOP SWITCH\n"); */
-
 }
 
 void
@@ -141,33 +137,12 @@ meta_compositor_destroy (MetaCompositor *compositor)
   clutter_threads_remove_repaint_func (compositor->repaint_func_id);
 }
 
-static void
-add_win (MetaWindow *window)
-{
-  MetaScreen		*screen = meta_window_get_screen (window);
-  MetaCompScreen        *info = meta_screen_get_compositor_data (screen);
-
-  g_return_if_fail (info != NULL);
-
-  meta_window_actor_new (window);
-
-  sync_actor_stacking (info);
-}
-
 static void
 process_damage (MetaCompositor     *compositor,
                 XDamageNotifyEvent *event,
                 MetaWindow         *window)
 {
-  MetaWindowActor *window_actor;
-
-  if (window == NULL)
-    return;
-
-  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  if (window_actor == NULL)
-    return;
-
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   meta_window_actor_process_x11_damage (window_actor, event);
 }
 
@@ -372,7 +347,6 @@ begin_modal_x11 (MetaScreen       *screen,
   Display        *xdpy        = meta_display_get_xdisplay (display);
   MetaCompScreen *info        = meta_screen_get_compositor_data (screen);
   Window          grab_window = clutter_x11_get_stage_window (CLUTTER_STAGE (info->stage));
-  Cursor          cursor      = None;
   int             result;
   gboolean        pointer_grabbed = FALSE;
   gboolean        keyboard_grabbed = FALSE;
@@ -392,7 +366,7 @@ begin_modal_x11 (MetaScreen       *screen,
                              META_VIRTUAL_CORE_POINTER_ID,
                              grab_window,
                              timestamp,
-                             cursor,
+                             None,
                              XIGrabModeAsync, XIGrabModeAsync,
                              False, /* owner_events */
                              &mask);
@@ -436,45 +410,6 @@ begin_modal_x11 (MetaScreen       *screen,
   return FALSE;
 }
 
-static gboolean
-begin_modal_wayland (MetaScreen       *screen,
-                     MetaPlugin       *plugin,
-                     MetaModalOptions  options,
-                     guint32           timestamp)
-{
-  MetaWaylandCompositor *compositor;
-  gboolean pointer_grabbed = FALSE;
-  gboolean keyboard_grabbed = FALSE;
-
-  compositor = meta_wayland_compositor_get_default ();
-
-  if ((options & META_MODAL_POINTER_ALREADY_GRABBED) == 0)
-    {
-      if (!meta_wayland_pointer_begin_modal (&compositor->seat->pointer))
-        goto fail;
-
-      pointer_grabbed = TRUE;
-    }
-  if ((options & META_MODAL_KEYBOARD_ALREADY_GRABBED) == 0)
-    {
-      if (!meta_wayland_keyboard_begin_modal (&compositor->seat->keyboard,
-                                              timestamp))
-        goto fail;
-
-      keyboard_grabbed = TRUE;
-    }
-
-  return TRUE;
-
- fail:
-  if (pointer_grabbed)
-    meta_wayland_pointer_end_modal (&compositor->seat->pointer);
-  if (keyboard_grabbed)
-    meta_wayland_keyboard_end_modal (&compositor->seat->keyboard, timestamp);
-
-  return FALSE;
-}
-
 gboolean
 meta_begin_modal_for_plugin (MetaScreen       *screen,
                              MetaPlugin       *plugin,
@@ -485,15 +420,14 @@ meta_begin_modal_for_plugin (MetaScreen       *screen,
    * are significant differences in how we handle grabs that make it difficult to
    * merge the two.
    */
-  MetaDisplay    *display    = meta_screen_get_display (screen);
-  MetaCompositor *compositor = display->compositor;
+  MetaDisplay *display = meta_screen_get_display (screen);
   gboolean ok;
 
-  if (compositor->modal_plugin != NULL || display->grab_op != META_GRAB_OP_NONE)
+  if (is_modal (display) || display->grab_op != META_GRAB_OP_NONE)
     return FALSE;
 
   if (meta_is_wayland_compositor ())
-    ok = begin_modal_wayland (screen, plugin, options, timestamp);
+    ok = TRUE;
   else
     ok = begin_modal_x11 (screen, plugin, options, timestamp);
   if (!ok)
@@ -505,8 +439,6 @@ meta_begin_modal_for_plugin (MetaScreen       *screen,
   display->grab_have_pointer = TRUE;
   display->grab_have_keyboard = TRUE;
 
-  compositor->modal_plugin = plugin;
-
   return TRUE;
 }
 
@@ -515,21 +447,12 @@ meta_end_modal_for_plugin (MetaScreen     *screen,
                            MetaPlugin     *plugin,
                            guint32         timestamp)
 {
-  MetaDisplay    *display    = meta_screen_get_display (screen);
-  Display        *xdpy = meta_display_get_xdisplay (display);
-  MetaCompositor *compositor = display->compositor;
+  MetaDisplay *display = meta_screen_get_display (screen);
+  Display *xdpy = meta_display_get_xdisplay (display);
 
-  g_return_if_fail (compositor->modal_plugin == plugin);
+  g_return_if_fail (is_modal (display));
 
-  if (meta_is_wayland_compositor ())
-    {
-      MetaWaylandCompositor *compositor = meta_wayland_compositor_get_default ();
-
-      meta_wayland_pointer_end_modal (&compositor->seat->pointer);
-      meta_wayland_keyboard_end_modal (&compositor->seat->keyboard,
-                                       timestamp);
-    }
-  else
+  if (!meta_is_wayland_compositor ())
     {
       XIUngrabDevice (xdpy, META_VIRTUAL_CORE_POINTER_ID, timestamp);
       XIUngrabDevice (xdpy, META_VIRTUAL_CORE_KEYBOARD_ID, timestamp);
@@ -540,27 +463,6 @@ meta_end_modal_for_plugin (MetaScreen     *screen,
   display->grab_screen = NULL;
   display->grab_have_pointer = FALSE;
   display->grab_have_keyboard = FALSE;
-
-  compositor->modal_plugin = NULL;
-}
-
-/* This is used when reloading plugins to make sure we don't have
- * a left-over modal grab for this screen.
- */
-void
-meta_check_end_modal (MetaScreen *screen)
-{
-  MetaDisplay    *display    = meta_screen_get_display (screen);
-  MetaCompositor *compositor = display->compositor;
-
-  if (compositor->modal_plugin &&
-      meta_plugin_get_screen (compositor->modal_plugin) == screen)
-    {
-      meta_end_modal_for_plugin (screen,
-                                   compositor->modal_plugin,
-
-                                 CurrentTime);
-    }
 }
 
 static void
@@ -762,18 +664,25 @@ meta_compositor_unmanage_screen (MetaCompositor *compositor,
     }
 }
 
-/*
- * Shapes the cow so that the given window is exposed,
- * when metaWindow is NULL it clears the shape again
+/**
+ * meta_shape_cow_for_window:
+ * @screen: A #MetaScreen
+ * @window: (allow-none): A #MetaWindow to shape the COW for
+ *
+ * Sets an bounding shape on the COW so that the given window
+ * is exposed. If @window is %NULL it clears the shape again.
+ *
+ * Used so we can unredirect windows, by shaping away the part
+ * of the COW, letting the raw window be seen through below.
  */
 static void
 meta_shape_cow_for_window (MetaScreen *screen,
-                           MetaWindow *metaWindow)
+                           MetaWindow *window)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   Display *xdisplay = meta_display_get_xdisplay (meta_screen_get_display (screen));
 
-  if (metaWindow == NULL)
+  if (window == NULL)
       XFixesSetWindowShapeRegion (xdisplay, info->output, ShapeBounding, 0, 0, None);
   else
     {
@@ -782,7 +691,7 @@ meta_shape_cow_for_window (MetaScreen *screen,
       int width, height;
       MetaRectangle rect;
 
-      meta_window_get_frame_rect (metaWindow, &rect);
+      meta_window_get_frame_rect (window, &rect);
 
       window_bounds.x = rect.x;
       window_bounds.y = rect.y;
@@ -833,11 +742,12 @@ meta_compositor_add_window (MetaCompositor    *compositor,
 {
   MetaScreen *screen = meta_window_get_screen (window);
   MetaDisplay *display = meta_screen_get_display (screen);
+  MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
-  DEBUG_TRACE ("meta_compositor_add_window\n");
   meta_error_trap_push (display);
 
-  add_win (window);
+  meta_window_actor_new (window);
+  sync_actor_stacking (info);
 
   meta_error_trap_pop (display);
 }
@@ -846,17 +756,9 @@ void
 meta_compositor_remove_window (MetaCompositor *compositor,
                                MetaWindow     *window)
 {
-  MetaWindowActor         *window_actor     = NULL;
-  MetaScreen *screen;
-  MetaCompScreen *info;
-
-  DEBUG_TRACE ("meta_compositor_remove_window\n");
-  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  if (!window_actor)
-    return;
-
-  screen = meta_window_get_screen (window);
-  info = meta_screen_get_compositor_data (screen);
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
+  MetaScreen *screen = meta_window_get_screen (window);
+  MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
   if (info->unredirected_window == window)
     set_unredirected_window (info, NULL);
@@ -869,13 +771,7 @@ meta_compositor_set_updates_frozen (MetaCompositor *compositor,
                                     MetaWindow     *window,
                                     gboolean        updates_frozen)
 {
-  MetaWindowActor *window_actor;
-
-  DEBUG_TRACE ("meta_compositor_set_updates_frozen\n");
-  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  if (!window_actor)
-    return;
-
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   meta_window_actor_set_updates_frozen (window_actor, updates_frozen);
 }
 
@@ -884,13 +780,7 @@ meta_compositor_queue_frame_drawn (MetaCompositor *compositor,
                                    MetaWindow     *window,
                                    gboolean        no_delay_frame)
 {
-  MetaWindowActor *window_actor;
-
-  DEBUG_TRACE ("meta_compositor_queue_frame_drawn\n");
-  window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  if (!window_actor)
-    return;
-
+  MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
   meta_window_actor_queue_frame_drawn (window_actor, no_delay_frame);
 }
 
@@ -1070,9 +960,9 @@ meta_compositor_process_event (MetaCompositor *compositor,
 
   info = meta_screen_get_compositor_data (screen);
 
-  if (compositor->modal_plugin && is_grabbed_event (compositor->display, event))
+  if (is_modal (display) && is_grabbed_event (compositor->display, event))
     {
-      _meta_plugin_xevent_filter (compositor->modal_plugin, event);
+      meta_plugin_manager_xevent_filter (info->plugin_mgr, event);
 
       /* We always consume events even if the plugin says it didn't handle them;
        * exclusive is exclusive */
@@ -1083,10 +973,7 @@ meta_compositor_process_event (MetaCompositor *compositor,
     maybe_spoof_event_as_stage_event (info, window, event);
 
   if (meta_plugin_manager_xevent_filter (info->plugin_mgr, event))
-    {
-      DEBUG_TRACE ("meta_compositor_process_event (filtered,window==NULL)\n");
-      return TRUE;
-    }
+    return TRUE;
 
   if (!meta_is_wayland_compositor () &&
       event->type == meta_display_get_damage_event_base (compositor->display) + XDamageNotify)
@@ -1100,7 +987,6 @@ meta_compositor_process_event (MetaCompositor *compositor,
           window = meta_display_lookup_x_window (compositor->display, xwin);
         }
 
-      DEBUG_TRACE ("meta_compositor_process_event (process_damage)\n");
       process_damage (compositor, (XDamageNotifyEvent *) event, window);
     }
 
@@ -1122,11 +1008,7 @@ meta_compositor_filter_keybinding (MetaCompositor *compositor,
                                    MetaKeyBinding *binding)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
-
-  if (info->plugin_mgr)
-    return meta_plugin_manager_filter_keybinding (info->plugin_mgr, binding);
-
-  return FALSE;
+  return meta_plugin_manager_filter_keybinding (info->plugin_mgr, binding);
 }
 
 void
@@ -1135,11 +1017,7 @@ meta_compositor_show_window (MetaCompositor *compositor,
                              MetaCompEffect  effect)
 {
   MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  DEBUG_TRACE ("meta_compositor_show_window\n");
-  if (!window_actor)
-    return;
-
-  meta_window_actor_show (window_actor, effect);
+ meta_window_actor_show (window_actor, effect);
 }
 
 void
@@ -1148,10 +1026,6 @@ meta_compositor_hide_window (MetaCompositor *compositor,
                              MetaCompEffect  effect)
 {
   MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  DEBUG_TRACE ("meta_compositor_hide_window\n");
-  if (!window_actor)
-    return;
-
   meta_window_actor_hide (window_actor, effect);
 }
 
@@ -1162,10 +1036,6 @@ meta_compositor_maximize_window (MetaCompositor    *compositor,
 				 MetaRectangle	   *new_rect)
 {
   MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  DEBUG_TRACE ("meta_compositor_maximize_window\n");
-  if (!window_actor)
-    return;
-
   meta_window_actor_maximize (window_actor, old_rect, new_rect);
 }
 
@@ -1176,10 +1046,6 @@ meta_compositor_unmaximize_window (MetaCompositor    *compositor,
 				   MetaRectangle     *new_rect)
 {
   MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  DEBUG_TRACE ("meta_compositor_unmaximize_window\n");
-  if (!window_actor)
-    return;
-
   meta_window_actor_unmaximize (window_actor, old_rect, new_rect);
 }
 
@@ -1197,18 +1063,14 @@ meta_compositor_switch_workspace (MetaCompositor     *compositor,
   to_indx   = meta_workspace_index (to);
   from_indx = meta_workspace_index (from);
 
-  DEBUG_TRACE ("meta_compositor_switch_workspace\n");
-
   if (!info) /* During startup before manage_screen() */
     return;
 
   info->switch_workspace_in_progress++;
 
-  if (!info->plugin_mgr ||
-      !meta_plugin_manager_switch_workspace (info->plugin_mgr,
-					       from_indx,
-					       to_indx,
-					       direction))
+  if (!meta_plugin_manager_switch_workspace (info->plugin_mgr,
+                                             from_indx, to_indx,
+                                             direction))
     {
       info->switch_workspace_in_progress--;
 
@@ -1317,8 +1179,6 @@ meta_compositor_sync_stack (MetaCompositor  *compositor,
   GList *old_stack;
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
 
-  DEBUG_TRACE ("meta_compositor_sync_stack\n");
-
   /* This is painful because hidden windows that we are in the process
    * of animating out of existence. They'll be at the bottom of the
    * stack of X windows, but we want to leave them in their old position
@@ -1407,15 +1267,6 @@ meta_compositor_sync_window_geometry (MetaCompositor *compositor,
                                       gboolean did_placement)
 {
   MetaWindowActor *window_actor = META_WINDOW_ACTOR (meta_window_get_compositor_private (window));
-  MetaScreen      *screen = meta_window_get_screen (window);
-  MetaCompScreen  *info = meta_screen_get_compositor_data (screen);
-
-  DEBUG_TRACE ("meta_compositor_sync_window_geometry\n");
-  g_return_if_fail (info);
-
-  if (!window_actor)
-    return;
-
   meta_window_actor_sync_actor_geometry (window_actor, did_placement);
 }
 
@@ -1447,7 +1298,6 @@ meta_compositor_sync_screen_size (MetaCompositor  *compositor,
       Display        *xdisplay;
       Window          xwin;
 
-      DEBUG_TRACE ("meta_compositor_sync_screen_size\n");
       g_return_if_fail (info);
 
       xdisplay = meta_display_get_xdisplay (display);
@@ -1751,10 +1601,6 @@ meta_compositor_show_tile_preview (MetaCompositor *compositor,
                                    int             tile_monitor_number)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
-
-  if (!info->plugin_mgr)
-    return;
-
   meta_plugin_manager_show_tile_preview (info->plugin_mgr,
                                          window, tile_rect, tile_monitor_number);
 }
@@ -1764,9 +1610,5 @@ meta_compositor_hide_tile_preview (MetaCompositor *compositor,
                                    MetaScreen     *screen)
 {
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
-
-  if (!info->plugin_mgr)
-    return;
-
   meta_plugin_manager_hide_tile_preview (info->plugin_mgr);
 }
