@@ -37,7 +37,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-#include "meta-wayland-surface.h"
+#include "wayland/meta-wayland-surface.h"
 
 static void meta_window_present_delete_dialog (MetaWindow *window,
                                                guint32     timestamp);
@@ -261,8 +261,7 @@ meta_window_present_delete_dialog (MetaWindow *window, guint32 timestamp)
         {
           MetaWindow *w = tmp->data;
 
-          if (w->xtransient_for == window->xwindow &&
-              w->res_class &&
+          if (w->transient_for == window && w->res_class &&
               g_ascii_strcasecmp (w->res_class, "mutter-dialog") == 0)
             {
               meta_window_activate (w, timestamp);
