@@ -39,6 +39,7 @@
 #include <gbm.h>
 
 #include <gdk/gdk.h>
+#include <gdk/gdkx.h>
 
 #include <X11/cursorfont.h>
 #include <X11/extensions/Xfixes.h>
@@ -46,9 +47,10 @@
 
 #include "meta-cursor-tracker-private.h"
 #include "screen-private.h"
-#include "meta-wayland-private.h"
 #include "monitor-private.h"
 
+#include "wayland/meta-wayland-private.h"
+
 #define META_WAYLAND_DEFAULT_CURSOR_HOTSPOT_X 7
 #define META_WAYLAND_DEFAULT_CURSOR_HOTSPOT_Y 4
 
@@ -569,6 +571,9 @@ make_wayland_cursor_tracker (MetaScreen *screen)
 
   compositor = meta_wayland_compositor_get_default ();
   compositor->seat->cursor_tracker = self;
+  meta_cursor_tracker_update_position (self,
+                                       wl_fixed_to_int (compositor->seat->pointer.x),
+                                       wl_fixed_to_int (compositor->seat->pointer.y));
 
 #if defined(CLUTTER_WINDOWING_EGL)
   if (clutter_check_windowing_backend (CLUTTER_WINDOWING_EGL))
@@ -1074,10 +1079,9 @@ get_pointer_position_gdk (int         *x,
   GdkScreen *gscreen;
 
   gmanager = gdk_display_get_device_manager (gdk_display_get_default ());
-  gdevice = gdk_device_manager_get_client_pointer (gmanager);
+  gdevice = gdk_x11_device_manager_lookup (gmanager, META_VIRTUAL_CORE_POINTER_ID);
 
-  if (x || y)
-    gdk_device_get_position (gdevice, &gscreen, x, y);
+  gdk_device_get_position (gdevice, &gscreen, x, y);
   if (mods)
     gdk_device_get_state (gdevice,
                           gdk_screen_get_root_window (gscreen),
@@ -1144,3 +1148,12 @@ meta_cursor_tracker_set_pointer_visible (MetaCursorTracker *tracker,
                           tracker->screen->xroot);
     }
 }
+
+void
+meta_cursor_tracker_force_update (MetaCursorTracker *tracker)
+{
+  g_assert (meta_is_wayland_compositor ());
+
+  update_hw_cursor (tracker);
+  sync_cursor (tracker);
+}
