@@ -39,11 +39,9 @@
 #include "workspace-private.h"
 #include "keybindings-private.h"
 #include "stack.h"
-#include "xprops.h"
 #include <meta/compositor.h>
 #include "mutter-enum-types.h"
 #include "core.h"
-#include "meta-wayland.h"
 #include "meta-cursor-tracker-private.h"
 
 #include <X11/extensions/Xinerama.h>
@@ -54,6 +52,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "x11/xprops.h"
+
 static char* get_screen_name (MetaDisplay *display,
                               int          number);
 
@@ -809,11 +809,8 @@ meta_screen_free (MetaScreen *screen,
   
   meta_display_grab (display);
 
-  if (screen->display->compositor)
-    {
-      meta_compositor_unmanage_screen (screen->display->compositor,
-				       screen);
-    }
+  meta_compositor_unmanage_screen (screen->display->compositor,
+                                   screen);
   
   meta_display_unmanage_windows_for_screen (display, screen, timestamp);
   
@@ -2523,10 +2520,9 @@ on_monitors_changed (MetaMonitorManager *manager,
                        &changes);
     }
 
-  if (screen->display->compositor)
-    meta_compositor_sync_screen_size (screen->display->compositor,
-				      screen,
-                                      screen->rect.width, screen->rect.height);
+  meta_compositor_sync_screen_size (screen->display->compositor,
+                                    screen,
+                                    screen->rect.width, screen->rect.height);
 
   /* Queue a resize on all the windows */
   meta_screen_foreach_window (screen, meta_screen_resize_func, 0);
