@@ -38,12 +38,15 @@
 #include "screen-private.h"
 #include <meta/util.h>
 #include "stack.h"
-#include "iconcache.h"
 #include <X11/Xutil.h>
 #include <cairo.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <clutter/clutter.h>
-#include "meta-wayland-types.h"
+
+#include "x11/iconcache.h"
+#include "x11/group-private.h"
+
+#include "wayland/meta-wayland-types.h"
 
 typedef struct _MetaWindowQueue MetaWindowQueue;
 
@@ -89,15 +92,13 @@ struct _MetaWindow
   char *desc; /* used in debug spew */
   char *title;
 
-  char *icon_name;
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
   MetaIconCache icon_cache;
   Pixmap wm_hints_pixmap;
   Pixmap wm_hints_mask;
-  
+
   MetaWindowType type;
-  Atom type_atom;
 
   /* NOTE these five are not in UTF-8, we just treat them as random
    * binary data
@@ -159,8 +160,8 @@ struct _MetaWindow
   /* Whether we're fullscreen */
   guint fullscreen : 1;
 
-  /* Whether the urgent flag of WM_HINTS is set */
-  guint wm_hints_urgent : 1;
+  /* Whether the window is marked as urgent */
+  guint urgent : 1;
 
   /* Whether we have to fullscreen after placement */
   guint fullscreen_after_placement : 1;
@@ -262,13 +263,6 @@ struct _MetaWindow
   guint has_move_func : 1;
   guint has_resize_func : 1;
   guint has_fullscreen_func : 1;
-  
-  /* Weird "_NET_WM_STATE_MODAL" flag */
-  guint wm_state_modal : 1;
-
-  /* TRUE if the client forced these on */
-  guint wm_state_skip_taskbar : 1;
-  guint wm_state_skip_pager : 1;
 
   /* Computed whether to skip taskbar or not */
   guint skip_taskbar : 1;
@@ -322,15 +316,6 @@ struct _MetaWindow
 
   /* Transient parent is a root window */
   guint transient_parent_is_root_window : 1;
-
-  /* Info on which props we got our attributes from */
-  guint using_net_wm_name              : 1; /* vs. plain wm_name */
-  guint using_net_wm_visible_name      : 1; /* tracked so we can clear it */
-  guint using_net_wm_icon_name         : 1; /* vs. plain wm_icon_name */
-  guint using_net_wm_visible_icon_name : 1; /* tracked so we can clear it */
-
-  /* icon props have changed */
-  guint need_reread_icon : 1;
   
   /* if TRUE, window was maximized at start of current grab op */
   guint shaken_loose : 1;
@@ -431,8 +416,6 @@ struct _MetaWindow
    * gives the position and size of the client window (i.e. ignoring
    * the frame).
    *
-   * Position valid if user_has_moved, size valid if user_has_resized
-   *
    * Position always in root coords, unlike window->rect.
    */
   MetaRectangle user_rect;
@@ -471,10 +454,9 @@ struct _MetaWindowClass
 {
   GObjectClass parent_class;
 
-  void (*workspace_changed) (MetaWindow *window, int  old_workspace);
-  void (*focus)             (MetaWindow *window);
-  void (*raised)            (MetaWindow *window);
-  void (*unmanaged)         (MetaWindow *window);
+  void (*get_default_skip_hints) (MetaWindow *window,
+                                  gboolean   *skip_taskbar_out,
+                                  gboolean   *skip_pager_out);
 };
 
 /* These differ from window->has_foo_func in that they consider
@@ -567,9 +549,6 @@ void        meta_window_change_workspace   (MetaWindow  *window,
 /* Return whether the window should be currently mapped */
 gboolean    meta_window_should_be_showing   (MetaWindow  *window);
 
-/* See warning in window.c about this function */
-gboolean    __window_is_terminal (MetaWindow *window);
-
 void        meta_window_update_struts      (MetaWindow  *window);
 
 /* this gets root coords */
@@ -634,12 +613,6 @@ void meta_window_show_menu (MetaWindow *window,
                             int         button,
                             guint32     timestamp);
 
-gboolean meta_window_titlebar_is_onscreen    (MetaWindow *window);
-void     meta_window_shove_titlebar_onscreen (MetaWindow *window);
-
-void meta_window_set_gravity (MetaWindow *window,
-                              int         gravity);
-
 #ifdef HAVE_XSYNC
 void meta_window_update_sync_request_counter (MetaWindow *window,
                                               gint64      new_counter_value);
@@ -684,9 +657,8 @@ void meta_window_update_layer (MetaWindow *window);
 
 void meta_window_recalc_features    (MetaWindow *window);
 
-/* recalc_window_type is x11 only, wayland does its thing and then calls type_changed */
-void meta_window_recalc_window_type (MetaWindow *window);
-void meta_window_type_changed       (MetaWindow *window);
+void meta_window_set_type (MetaWindow     *window,
+                           MetaWindowType  type);
 
 void meta_window_frame_size_changed (MetaWindow *window);
 
@@ -751,4 +723,11 @@ void meta_window_activate_full (MetaWindow     *window,
                                 MetaClientType  source_indication,
                                 MetaWorkspace  *workspace);
 
+gboolean meta_window_is_client_decorated (MetaWindow *window);
+
+void meta_window_update_monitor (MetaWindow *window);
+
+void meta_window_set_urgent (MetaWindow *window,
+                             gboolean    urgent);
+
 #endif
