@@ -78,8 +78,7 @@ pointer_handle_focus_resource_destroy (struct wl_listener *listener, void *data)
 
 static void
 default_grab_focus (MetaWaylandPointerGrab *grab,
-                    MetaWaylandSurface     *surface,
-		    const ClutterEvent     *event)
+                    MetaWaylandSurface     *surface)
 {
   MetaWaylandPointer *pointer = grab->pointer;
 
@@ -385,7 +384,7 @@ meta_wayland_pointer_start_grab (MetaWaylandPointer *pointer,
   grab->pointer = pointer;
 
   if (pointer->current)
-    interface->focus (pointer->grab, pointer->current, NULL);
+    interface->focus (pointer->grab, pointer->current);
 }
 
 void
@@ -395,62 +394,7 @@ meta_wayland_pointer_end_grab (MetaWaylandPointer *pointer)
 
   pointer->grab = &pointer->default_grab;
   interface = pointer->grab->interface;
-  interface->focus (pointer->grab, pointer->current, NULL);
-}
-
-static void
-modal_focus (MetaWaylandPointerGrab *grab,
-	     MetaWaylandSurface     *surface,
-	     const ClutterEvent     *event)
-{
-}
-
-static void
-modal_motion (MetaWaylandPointerGrab *grab,
-	      const ClutterEvent     *event)
-{
-}
-
-static void
-modal_button (MetaWaylandPointerGrab *grab,
-	      const ClutterEvent     *event)
-{
-}
-
-static MetaWaylandPointerGrabInterface modal_grab = {
-  modal_focus,
-  modal_motion,
-  modal_button
-};
-
-gboolean
-meta_wayland_pointer_begin_modal (MetaWaylandPointer *pointer)
-{
-  MetaWaylandPointerGrab *grab;
-
-  if (pointer->grab != &pointer->default_grab)
-    return FALSE;
-
-  meta_wayland_pointer_set_focus (pointer, NULL);
-
-  grab = g_slice_new0 (MetaWaylandPointerGrab);
-  grab->interface = &modal_grab;
-  meta_wayland_pointer_start_grab (pointer, grab);
-
-  return TRUE;
-}
-
-void
-meta_wayland_pointer_end_modal (MetaWaylandPointer *pointer)
-{
-  MetaWaylandPointerGrab *grab;
-
-  grab = pointer->grab;
-
-  g_assert (grab->interface == &modal_grab);
-
-  meta_wayland_pointer_end_grab (pointer);
-  g_slice_free (MetaWaylandPointerGrab, grab);
+  interface->focus (pointer->grab, pointer->current);
 }
 
 typedef struct {
@@ -470,15 +414,14 @@ typedef struct {
 
 static void
 popup_grab_focus (MetaWaylandPointerGrab *grab,
-		  MetaWaylandSurface     *surface,
-		  const ClutterEvent     *event)
+		  MetaWaylandSurface     *surface)
 {
   MetaWaylandPopupGrab *popup_grab = (MetaWaylandPopupGrab*)grab;
 
   /* Popup grabs are in owner-events mode (ie, events for the same client
      are reported as normal) */
   if (surface && wl_resource_get_client (surface->resource) == popup_grab->grab_client)
-    default_grab_focus (grab, surface, event);
+    default_grab_focus (grab, surface);
   else
     meta_wayland_pointer_set_focus (grab->pointer, NULL);
 }
@@ -533,6 +476,12 @@ meta_wayland_pointer_end_popup_grab (MetaWaylandPointer *pointer)
       g_slice_free (MetaWaylandPopup, popup);
     }
 
+  {
+    MetaDisplay *display = meta_get_display ();
+    meta_display_end_grab_op (display,
+                              meta_display_get_current_time_roundtrip (display));
+  }
+
   meta_wayland_pointer_end_grab (pointer);
   g_slice_free (MetaWaylandPopupGrab, popup_grab);
 }
@@ -572,6 +521,8 @@ meta_wayland_pointer_start_popup_grab (MetaWaylandPointer *pointer,
 
   if (pointer->grab == &pointer->default_grab)
     {
+      MetaWindow *window = surface->window;
+
       grab = g_slice_new0 (MetaWaylandPopupGrab);
       grab->generic.interface = &popup_grab_interface;
       grab->generic.pointer = pointer;
@@ -579,6 +530,19 @@ meta_wayland_pointer_start_popup_grab (MetaWaylandPointer *pointer,
       wl_list_init (&grab->all_popups);
 
       meta_wayland_pointer_start_grab (pointer, (MetaWaylandPointerGrab*)grab);
+
+      meta_display_begin_grab_op (window->display,
+                                  window->screen,
+                                  window,
+                                  META_GRAB_OP_WAYLAND_CLIENT,
+                                  FALSE, /* pointer_already_grabbed */
+                                  FALSE, /* frame_action */
+                                  1, /* button. XXX? */
+                                  0, /* modmask */
+                                  meta_display_get_current_time_roundtrip (window->display),
+                                  wl_fixed_to_int (pointer->grab_x),
+                                  wl_fixed_to_int (pointer->grab_y));
+
     }
   else
     grab = (MetaWaylandPopupGrab*)pointer->grab;
@@ -593,7 +557,6 @@ meta_wayland_pointer_start_popup_grab (MetaWaylandPointer *pointer,
     wl_resource_add_destroy_listener (surface->wl_shell_surface.resource, &popup->surface_destroy_listener);
 
   wl_list_insert (&grab->all_popups, &popup->link);
-      
   return TRUE;
 }
 
