@@ -508,6 +508,15 @@ stage_destroy_cb (void)
   meta_quit (META_EXIT_SUCCESS);
 }
 
+/**
+ * meta_wayland_compositor_update:
+ * @compositor: the #MetaWaylandCompositor instance
+ * @event: the #ClutterEvent used to update @seat's state
+ *
+ * This is used to update display server state like updating cursor
+ * position and keeping track of buttons and keys pressed. It must be
+ * called for all input events coming from the underlying devices.
+ */
 void
 meta_wayland_compositor_update (MetaWaylandCompositor *compositor,
                                 const ClutterEvent    *event)
@@ -536,18 +545,18 @@ meta_wayland_compositor_update (MetaWaylandCompositor *compositor,
       meta_idle_monitor_reset_idletime (device_monitor);
     }
 
-  switch (event->type)
-    {
-    case CLUTTER_MOTION:
-    case CLUTTER_BUTTON_PRESS:
-    case CLUTTER_BUTTON_RELEASE:
-    case CLUTTER_SCROLL:
-      meta_wayland_seat_update_pointer (compositor->seat, event);
-    default:
-      break;
-    }
+  meta_wayland_seat_update (compositor->seat, event);
 }
 
+/**
+ * meta_wayland_compositor_handle_event:
+ * @compositor: the #MetaWaylandCompositor instance
+ * @event: the #ClutterEvent to be sent
+ *
+ * This method sends events to the focused wayland client, if any.
+ *
+ * Return value: whether @event was sent to a wayland client.
+ */
 gboolean
 meta_wayland_compositor_handle_event (MetaWaylandCompositor *compositor,
                                       const ClutterEvent    *event)
@@ -643,6 +652,8 @@ meta_wayland_init (void)
 
   clutter_wayland_set_compositor_display (compositor->wayland_display);
 
+  /* If we're running on bare metal, we're a display server,
+   * so start talking to weston-launch. */
 #if defined(CLUTTER_WINDOWING_EGL)
   if (clutter_check_windowing_backend (CLUTTER_WINDOWING_EGL))
     compositor->launcher = meta_launcher_new ();
@@ -651,17 +662,6 @@ meta_wayland_init (void)
   if (clutter_init (NULL, NULL) != CLUTTER_INIT_SUCCESS)
     g_error ("Failed to initialize Clutter");
 
-#if defined(CLUTTER_WINDOWING_EGL)
-  if (clutter_check_windowing_backend (CLUTTER_WINDOWING_EGL))
-    {
-      ClutterBackend *backend = clutter_get_default_backend ();
-      CoglContext *cogl_context = clutter_backend_get_cogl_context (backend);
-      CoglRenderer *cogl_renderer = cogl_display_get_renderer (cogl_context_get_display (cogl_context));
-      int drm_fd = cogl_kms_renderer_get_kms_fd (cogl_renderer);
-      meta_launcher_set_drm_fd (compositor->launcher, drm_fd, NULL);
-    }
-#endif
-
   meta_monitor_manager_initialize ();
   monitors = meta_monitor_manager_get ();
   g_signal_connect (monitors, "monitors-changed",
@@ -728,7 +728,22 @@ meta_wayland_compositor_activate_vt (MetaWaylandCompositor  *compositor,
     }
   else
     {
-      g_debug ("Ignoring VT switch keybinding, not running as VT manager");
+      g_debug ("Ignoring VT switch keybinding, not running as display server");
+      return TRUE;
+    }
+}
+
+gboolean
+meta_wayland_compositor_activate_session (MetaWaylandCompositor  *compositor,
+                                          GError                **error)
+{
+  if (compositor->launcher)
+    {
+      return meta_launcher_activate_vt (compositor->launcher, -1, error);
+    }
+  else
+    {
+      g_debug ("Ignoring activate_session, not running as display server");
       return TRUE;
     }
 }
